function [A,F]=assem(assKey, parm, UU, VERT, SIMP);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% FEMBIF = < Finite Element Methods for Bifurcation >
%% Copyright (C) 1997  Michael Holst
%%
%% This program is free software; you can redistribute it and/or modify it
%% under the terms of the GNU General Public License as published by the Free
%% Software Foundation; either version 2 of the License, or (at your option)
%% any later version.
%%
%% This program is distributed in the hope that it will be useful, but WITHOUT
%% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
%% FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
%% more details.
%%
%% You should have received a copy of the GNU General Public License along
%% with this program; if not, write to the Free Software Foundation, Inc.,
%% 675 Mass Ave, Cambridge, MA 02139, USA.
%%
%% Typing "help 0license" should list the GNU GPL to the screen.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Prototype:
%
%    [A,F]=assem(assKey, parm, UU, VERT, SIMP);
%
% Purpose:
%
%    Finite element matrix assembly.
%
% Input:
%
%    (N  = number of vertices in mesh)
%    (L  = number of simplices in mesh)
%
%    assKey = assembly path
%    parm   = pde parameters
%    UU     = zeros(N,1); current finite element solution
%    VERT   = zeros(N,6); vertices, (x,y,z)-coordinates of all vertices
%    SIMP   = zeros(L,9); simplices, vertices numbers given counter-clockwise
%
% Output:
%
%    assKey==0 ==>    A = zeros(N,N) = linear stiffness matrix
%                     F = zeros(N,1) = linear load vector
%
%    assKey==1 ==>    A = zeros(N,N) = nonlinear tangent stiffness matrix
%                     F = zeros(N,1) = linear load vector
%
%    assKey==2 ==>    A = zeros(N,N) = nonlinear tangent stiffness matrix
%                     F = zeros(N,1) = nonlinear residual vector
%
%    assKey==3 ==>    A = zeros(N,N) = (not constructed)
%                     F = zeros(N,1) = nonlinear residual vector
%
%
% Other Variables:
%
%    T  = number of vertices in a single element
%    Q  = number of quadrature points in the interior of the master element
%    QE = number of quadrature points along the master element edges
%
%    INTPT   = zeros(Q,2)  = master element volume integration points
%    W       = zeros(Q,1)  = master element volume integration weights
%    PHI     = zeros(T,Q)  = values of local basis funcs at integ pts
%    PHIX    = zeros(T,Q)  = values of x-derivs of local basis funcs integ pts
%    PHIY    = zeros(T,Q)  = values of y-derivs of local basis funcs integ pts
%
%    INTPTE1 = zeros(QE,2) = type (2,3) EDGE integration points
%    WE1     = zeros(QE,1) = type (2,3) EDGE integration weights
%    PHIE1   = zeros(T,QE) = type (2,3) basis function values at EDGE integ pts
%
%    INTPTE2 = zeros(QE,2) = type (3,1) EDGE integration points
%    WE2     = zeros(QE,1) = type (3,1) EDGE integration weights
%    PHIE2   = zeros(T,QE) = type (3,1) basis function values at EDGE integ pts
%
%    INTPTE3 = zeros(QE,2) = type (1,2) EDGE integration points
%    WE3     = zeros(QE,1) = type (1,2) EDGE integration weights
%    PHIE3   = zeros(T,QE) = type (1,2) basis function values at EDGE integ pts
%
% Author:   Michael Holst 050497
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   %%% First recover various problem dimensions.
   [N,six]  = size(VERT);
   [L,nine] = size(SIMP);
   T=3;

   %%% Get master element info.
   [INTPT,W,PHI,PHIX,PHIY, ...
    INTPTE1,WE1,PHIE1, ...
    INTPTE2,WE2,PHIE2, ...
    INTPTE3,WE3,PHIE3] = mastr(0);
   [Q,two]  = size(INTPT);
   [QE,two] = size(INTPTE1);

   %%% Initialize global stiffness matrix and global load vector.
   A = spalloc(N,N,0);  
   F = zeros(N,1);

   %%% Get coordinates of vertices of each element.
   Xi0(1:L,1) = VERT(SIMP(1:L,1),1);
   Xi1(1:L,1) = VERT(SIMP(1:L,2),1);
   Xi2(1:L,1) = VERT(SIMP(1:L,3),1);
   Yi0(1:L,1) = VERT(SIMP(1:L,1),2);
   Yi1(1:L,1) = VERT(SIMP(1:L,2),2);
   Yi2(1:L,1) = VERT(SIMP(1:L,3),2);

   %%% Build affine transformations (from master element to arbitrary).
   f00(1:L,1) = Xi1(1:L,1) - Xi0(1:L,1);
   f01(1:L,1) = Xi2(1:L,1) - Xi0(1:L,1);
   f10(1:L,1) = Yi1(1:L,1) - Yi0(1:L,1);
   f11(1:L,1) = Yi2(1:L,1) - Yi0(1:L,1);
   b0(1:L,1)  = Xi0(1:L,1);
   b1(1:L,1)  = Yi0(1:L,1);

   %%% Build the jacobians.
   D (1:L,1) = abs( f00 .* f11 - f01 .* f10 );
   Di(1:L,1) = 1.0 ./ D(1:L,1);

   %%% Build inverse transformations (from arbitrary element to master).
   g00(1:L,1) =  Di(1:L,1) .* f11(1:L,1);
   g01(1:L,1) = -Di(1:L,1) .* f01(1:L,1);
   g10(1:L,1) = -Di(1:L,1) .* f10(1:L,1);
   g11(1:L,1) =  Di(1:L,1) .* f00(1:L,1);

   %%% face types
   ftype = facetype(SIMP(1:L,5));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Build the stiffness matrix and load vector at the same time.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   %%% (Go through all simplices AT ONCE and do element integrals)

   %%% At each r loop iter, compute el stiffness matrix and el load vec entries
   for r=1:T

      %%% initialize element load entry fr
      fr(1:L,1) = zeros(L,1);

      %%% At each s loop iter, compute el stiffness matrix entries
      for s=1:T

         %%% initialize element matrix entry ars 
         ars(1:L,1) = zeros(L,1);

         %%% Go thru quad pts m=1:Q, computing contrib. to element matrix.
         for m=1:Q

            %%% Get quad pts by mapping master Gauss pts to our element
            xm(1:L,1)=f00(1:L,1)*INTPT(m,1)+f01(1:L,1)*INTPT(m,2)+b0(1:L,1);
            ym(1:L,1)=f10(1:L,1)*INTPT(m,1)+f11(1:L,1)*INTPT(m,2)+b1(1:L,1);

            %%% We must evaluate function at the current quadrature point m
            for q=1:T
               p(q) = PHI(q,m);
               px(1:L,q) = g00(1:L,1) * PHIX(q,m) + g10(1:L,1) * PHIY(q,m);
               py(1:L,q) = g01(1:L,1) * PHIX(q,m) + g11(1:L,1) * PHIY(q,m);
            end

            %%% current solution at quadrature points in all simplices
            U(1:L,1) = p(1) * UU( SIMP(1:L,1), 1 ) ...
                     + p(2) * UU( SIMP(1:L,2), 1 ) ...
                     + p(3) * UU( SIMP(1:L,3), 1 );
            UX(1:L,1) = px(1:L,1) .* UU( SIMP(1:L,1), 1 ) ...
                      + px(1:L,2) .* UU( SIMP(1:L,2), 1 ) ...
                      + px(1:L,3) .* UU( SIMP(1:L,3), 1 );
            UY(1:L,1) = py(1:L,1) .* UU( SIMP(1:L,1), 1 ) ...
                      + py(1:L,2) .* UU( SIMP(1:L,2), 1 ) ...
                      + py(1:L,3) .* UU( SIMP(1:L,3), 1 );

            %%% do the load vector while we are here...
            if (s==1)

               %%% Evaluate nonlinear function at quadrature points
               evalKey = 0;
               thetam = fu_v(evalKey,parm,L, xm,ym, U,UX,UY, ...
                  p(r), px(1:L,r),py(1:L,r));

               %%% Do quad using weights W
               fr(1:L,1) = fr(1:L,1) + W(m) * (D(1:L,1) .* thetam(1:L,1));
            end;

            %%% Evaluate bilinear form at quadrature points
            evalKey = 0;
            thetam = dfu_wv(evalKey,parm,L, xm,ym, U,UX,UY, ...
                            p(s),px(1:L,s),py(1:L,s), ...
                            p(r),px(1:L,r),py(1:L,r));

            %%% Do quad using weights W
            ars(1:L,1) = ars(1:L,1) + W(m) * (D(1:L,1) .* thetam(1:L,1));
         end

         %%% Add contribution to global stiffness matrix.
         i(1:L,1)  = SIMP(1:L,r);
         j(1:L,1)  = SIMP(1:L,s);
         ik(1:L,1) = VERT( SIMP(1:L,r), 4 );
         jk(1:L,1) = VERT( SIMP(1:L,s), 4 );
         for l=1:L
            if ( (ik(l) ~= 1) & (jk(l) ~= 1) )
               A(i(l),j(l)) = A(i(l),j(l)) + ars(l);
            else
               if (ik(l) == 1)
                  A(i(l),i(l)) = 1.0;
               end
               if (jk(l) == 1)
                  A(j(l),j(l)) = 1.0;
               end
            end
         end

      end

      %%% Add contribution to global load vector
      i(1:L,1)  = SIMP(1:L,r);
      ik(1:L,1) = VERT( SIMP(1:L,r), 4 );
      for l=1:L
         if (ik(l) ~= 1)
            F(i(l)) = F(i(l)) - fr(l);
         else
            if (assKey <= 1)
               F(i(l)) = u_d(VERT(i(l),1),VERT(i(l),2));
            else
               F(i(l)) = 0.;
            end
         end
      end
   end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Handle the natural (Neumann) boundary conditions.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Compute total time and return.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end

